/************************ SETIS Automao e Sistemas **************************

 File            : PTI_DLL.h
 Project         : Payment Terminal Integration
 Platform        : Win32
 Creation date   : Jul/22/2014
 Author          : Guilherme Eduardo Leite
 Description     : Definitions exported by the Payment Terminal Integration Library
 ================================= HISTORY ====================================
 Date        Author         Change description
 --------- -------------- -----------------------------------------------------
 22/Nov/16 Thomaz Capra  - Includa funo PTI_PrnSymbolCode e PTIINF_RCPTSHORT
                           PTIINF_RCPTFULL.
                         - Removido defines de tipos quando compilado pelo VS.
 05/Jan/16 Thomaz Capra  - Includo denfine CODESYMB_128.

 \*****************************************************************************/

#ifndef _PTI_DLL_H
#define _PTI_DLL_H

/* Tipos de dados j esto definidos nos headers do Windows */
/* https://msdn.microsoft.com/en-us/library/windows/desktop/aa383751(v=vs.85).aspx */
#ifdef _MSC_VER 
#include <BaseTsd.h>
#include <WinDef.h>
#include <WinNT.h>
#endif //_MSC_VER

/*************/
/* Constants */
/*************/

/* Return codes */
#define PTIRET_OK              0
#define PTIRET_INVPARAM       -2001
#define PTIRET_NOCONN         -2002
#define PTIRET_BUSY           -2003
#define PTIRET_TIMEOUT        -2004
#define PTIRET_CANCEL         -2005
#define PTIRET_NODATA         -2006
#define PTIRET_BUFOVRFLW      -2007
#define PTIRET_SOCKETERR      -2008
#define PTIRET_WRITEERR       -2009
#define PTIRET_EFTERR         -2010
#define PTIRET_INTERNALERR    -2011
#define PTIRET_PROTOCOLERR    -2012
#define PTIRET_SECURITYERR    -2013
#define PTIRET_PRINTERR       -2014
#define PTIRET_NOPAPER        -2015
#define PTIRET_NEWCONN	      -2016
#define PTIRET_NONEWCONN	   -2017
#define PTIRET_NOTSUPORTED    -2057
#define PTIRET_CRYPTERR       -2058
 
/* Terminal status */
#define PTISTAT_IDLE       0
#define PTISTAT_BUSY       1
#define PTISTAT_NOCONN     2
#define PTISTAT_WAITRECON  3

/* Transaction types     
#define PTITRN_SALE       1
#define PTITRN_ADMIN      2
#define PTITRN_SALEVOID   4*/

/* Beep types */
#define PTIBEEP_OK        0
#define PTIBEEP_WARNING   1
#define PTIBEEP_ERROR     2
         
/* Transaction information */
// #define PTI_RESULT              100

/* Final status for the transaction */
#define PTICNF_SUCCESS     1
#define PTICNF_PRINTERR    2
#define PTICNF_DISPFAIL    3
#define PTICNF_OTHERERR    4
#define PTICNF_COMERR      5

/* Receipt copies */
#define PTIPRN_MERCHANT	   1
#define PTIPRN_CHOLDER	   2

/* Tipo de cdigos QR/BAR */
#define CODESYMB_EAN13     1 /* no implementado */
#define CODESYMB_128       2
#define CODESYMB_ITF14     3
#define CODESYMB_QRCODE    4

/* Definicoes das teclas do POS */
#define PTIKEY_0        '0'
#define PTIKEY_1        '1'
#define PTIKEY_2        '2'
#define PTIKEY_3        '3'
#define PTIKEY_4        '4'
#define PTIKEY_5        '5'
#define PTIKEY_6        '6'
#define PTIKEY_7        '7'
#define PTIKEY_8        '8'
#define PTIKEY_9        '9'
#define PTIKEY_STAR     '*'
#define PTIKEY_HASH     '#'
#define PTIKEY_DOT      '.'
#define PTIKEY_00       37
#define PTIKEY_BACKSP   8
#define PTIKEY_OK       13
#define PTIKEY_CANCEL   27
#define PTIKEY_FUNC0    97
#define PTIKEY_FUNC1    98
#define PTIKEY_FUNC2    99
#define PTIKEY_FUNC3    100
#define PTIKEY_FUNC4    101
#define PTIKEY_FUNC5	   102
#define PTIKEY_FUNC6	   103
#define PTIKEY_FUNC7	   104
#define PTIKEY_FUNC8	   105
#define PTIKEY_FUNC9	   106
#define PTIKEY_FUNC10	107
#define PTIKEY_TOUCH    126
#define PTIKEY_ALPHA    38

/* Somente ir definir os tipos abaixo fora do VS */
#ifndef _MSC_VER 
#ifndef UINT16
#define UINT16 unsigned short 
#endif /* UINT16 */

#ifndef PUINT16 
#define PUINT16 unsigned short*
#endif /* PUINT16 */

#ifndef INT16  
#define INT16 short
#endif /* INT16 */

#ifndef PINT16
#define PINT16 short*
#endif /* PINT16 */

#ifndef LPCSTR
#define LPCSTR const char*
#endif /* LPCSTR */

#ifndef LPSTR
#define LPSTR char*
#endif /* LPSTR */

#ifndef BOOL
#define BOOL char
#endif /* BOOL */

#ifndef PBYTE
#define PBYTE unsigned char*
#endif /* PBYTE */
#endif

/**********************/
/* Exported functions */
/**********************/
/*===============================================================================================*\
 Function     :  PTI_Init

 Description  :  This function configures the integration library and should be the  first one called 
                 by the POS software. The integration library will only accept payment terminal 
                 connections after it is called.
 
 Input         :  pszPOS_Company Name of the POS software company (null-terminated, up to 40 characters, 
                                 no accented characters). For example, "ABC SYSTEMS".

                  pszPOS_Version Name and version of the POS software (null-terminated, up to 40 characters, 
                                 no accented characters). For example, POWERSALES v1.01.

                  pszPOS_Capabilities  Workability of automation.

                  pszDataFolder  Full path of the directory for storing integration librarys data and logs. 
                                 Note: The operating system user that runs the POS software executable s
                                 hould have write permissions in this directory.

                  uiTCP_Port     Local listening TCP port that all payment terminals will connect to. Note: 
                                 this port should be enabled for incoming connections on any firewall that 
                                 stands between the POS software and the payment terminals.
   
                  uiMaxTerminals Maximum number of terminals that are allowed to connect to the POS software.

                  pszWaitMsg     Message to be displayed by any terminal immediately after it goes online. 
                                 See PTI_Display for format information.

                  uiAutoDiscSec  Number of seconds after which the terminal should go offline when running on 
                                 battery, or zero to never go offline. See PTI_Disconnect for additional 
                                 information.
 
 Output        :  none.
 
 Return        :  PTIRET_OK         Successful operation.
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
                  PTIRET_SOCKETERR  Unable to start listening to the specified TCP port.
                  PTIRET_WRITEERR   Unable to use the specified directory.
\*===============================================================================================*/
extern void PTI_Init (LPCSTR pszPOS_Company, LPCSTR pszPOS_Version, LPCSTR pszPOS_Capabilities, 
                      LPCSTR pszDataFolder, UINT16 uiTCP_Port, UINT16 uiMaxTerminals, 
                      LPCSTR pszWaitMsg, UINT16 uiAutoDiscSec, PINT16 piRet);

/*===============================================================================================*\
 Function      :  PTI_End

 Description   :  This function must be the last function called by the POS software, when terminated
                  or before unloading the integration library. At this moment, the integration library 
                  releases all allocated resources (TCP ports, processes, memory, etc.).
 
 Input         :  none.
 
 Output        :  none.
 
 Return        :  none.
\*===============================================================================================*/
extern void PTI_End(void);

/*===============================================================================================*\
 Function      :  PTI_CheckStatus

 Description   :  This function allows the POS software to check the status (online or offline) of 
                  a specific payment terminal, and retrieve additional information from the device.
                  Each payment terminal is given a unique logical identifier, which is configured 
                  when the terminal is set up. If the POS software controls more than one terminal, 
                  it should have a record of all identifiers and their location, in order to be able 
                  to send the payment commands to the intended terminal.
 
 Input         :  pszTerminalId  Terminal unique identifier (null-terminated). May be empty if the 
                                 maximum supported number of terminals (informed in PTI_Init) is 1.
 
 Output        :  pszTerminalId  Terminal unique identifier (null-terminated, up to 20 characters).

                  piStatus       Terminal status (PTISTAT_xxx).

                  pszModel       Terminal model (null-terminated, up to 20 characters).

                  pszMAC         Terminal MAC address (null-terminated, format XX:XX:XX:XX:XX:XX).

                  pszSerNo       Terminal serial number (null-terminated, up to 25 characters).
 
 Return        :  PTIRET_OK      Operao bem sucedida.
\*===============================================================================================*/
extern void PTI_CheckStatus (LPCSTR pszTerminalId, PINT16 piStatus, LPSTR pszModel, LPSTR pszMAC, 
                             LPSTR pszSerNo, PINT16 piRet);

/*===============================================================================================*\
 Function      :  PTI_Disconnect

 Description   :  This function allows the POS software to disconnect a payment terminal and put it 
                  in offline mode, either immediately or after some time running on battery. For mobile 
                  terminals, remaining online significantly decreases the battery charge. It is 
                  therefore recommended that the POS software specify a non-zero value for the 
                  uiAutoDiscSec parameter from PTI_Init, or call this function as soon as the terminal 
                  connects.After the terminal goes offline, as soon as a key is pressed, it automatically 
                  connects again to the POS software.
 
 Input         :  pszTerminalId  Terminal unique identifier (null-terminated).

                  uiPwrDelay     If zero, immediately disconnects the terminal, independently from 
                                 its power source. If non-zero, represents the maximum number of 
                                 seconds during which the terminal will remain online while on battery. 
                                 The terminal will not go offline while connected to the mains. Overwrites 
                                 the uiAutoDiscSec parameter from PTI_Init for this specific terminal.
 
 Output        :  none.
 
 Return        :  PTIRET_OK      Successful operation.
                  PTIRET_NOCONN  The terminal is offline.
                  PTIRET_BUSY    The terminal is busy processing another command.

\*===============================================================================================*/
extern void PTI_Disconnect (LPCSTR pszTerminalId, UINT16 uiPwrDelay, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_Display

Description    :  This function puts a message on the terminals display, and returns immediately.
                  The message is displayed from the upper left corner of the screen, 20 characters 
                  per line, with line breaks identified by the '\r' character (carriage return, ASCII 
                  code 13). Characters beyond 20 columns or the maximum amount of rows are discarded. 
                  The maximum number of supported lines may vary according to the terminal model, 
                  but a minimum of four rows is always supported.
 
Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

                  pszMsg         Message to be displayed on the terminal (null-terminated).

Output         :  none.

Return         :  PTIRET_OK            Successful operation.
                  PTIRET_INVPARAM      Invalid parameter passed to the function.
                  PTIRET_NOCONN        The terminal is offline.
                  PTIRET_BUSY          The terminal is busy processing another command.
                  PTIRET_NOTSUPORTED   Function not supported by the terminal.
\*===============================================================================================*/
extern void PTI_Display (LPCSTR pszTerminalId, LPCSTR pszMsg, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_WaitKey

Description    :  This function waits for a keystroke on the terminal, and only returns after a 
                  key is pressed or a time-out occurs.                  
 
Input          :  pszTerminalId  Terminal unique identifier (null-terminated).
                  uiTimeOutSec   User time-out, in seconds. If equal to zero, the function 
                                 returns immediately, only reporting a key if it was pressed 
                                 before the function was called. (Keystrokes are buffered).

Output         :  piKey          Identifier of the key that was pressed, according to table below 
                                 (only if the function returns PTIRET_OK).

Return         :  PTIRET_OK            Successful operation.
                  PTIRET_NOCONN        The terminal is offline.
                  PTIRET_BUSY          The terminal is busy processing another command.
                  PTIRET_TIMEOUT       No key was pressed during the specified time.

 Note: This function should only be used to capture isolated keystrokes, it should not be called 
       successively in order to perform a data entry. For this purpose, use PTI_GetData instead.
\*===============================================================================================*/
extern void PTI_WaitKey (LPCSTR pszTerminalId, UINT16 uiTimeOutSec, PINT16 piKey, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_ClearKey

Description    :  This function clears the pressed key buffer, so that the next function call 
                  does not consider any previously pressed key. This function returns immediately.
 
Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

Output         :  none.

Return         :  PTIRET_OK            Successful operation.
                  PTIRET_NOCONN        The terminal is offline.
                  PTIRET_BUSY          The terminal is busy processing another command.
\*===============================================================================================*/
extern void PTI_ClearKey (LPCSTR pszTerminalId, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_GetData

Description    :  This function performs a single data entry on a previsously connected terminal.
                  This function is blocking and only returns after the data entry is successful or 
                  fails.
 
Input          :  pszTerminalId  Terminal unique identifier (null-terminated).
                  pszPrompt      Null-terminated text message to be displayed to the user, 
                                 describing the information to be typed. Use \r (carriage return, 
                                 ASCII code 13) for line breaks. Example: TIP AMOUNT:
                  pszFormat      Null-terminated formatting mask. Use @ (at sign) for editable 
                                 character positions. Example: @@,@@@,@@@.@@ for an amount in 
                                 cents. May be NULL or empty for direct entry without a format.
                  uiLenMin       Minimum number of characters.
                  uiLenMax       Maximum number of characters.
                  fFromLeft      TRUE  (1) to start typing from the left; 
                                 FALSE (0) to start typing from the right.
                  fAlpha         TRUE (1) to enable non-numeric characters; 
                                 FALSE (0) to allow only numeric characters.
                                 Note: as typing non-numeric characters on most terminal devices is not 
                                 user-friendly, please avoid the use of this feature whenever possible.
                  fMask          TRUE (1) to mask the typed characters with an asterisk (tipically 
                                 for a password entry); FALSE (0) to show the typed characters.
                  uiTimeOutSec   Maximum time between to keystrokes, in seconds.
                  pszData        Null-terminated initial value for the data to be edited.
                  uiCaptureLine  Display line index (starting from 1) where the typed information 
                                 should be displayed. If the prompt display message is also displayed 
                                 on this line, the typed information will be displayed right after 
                                 the prompt; else, it will be displayed starting from the first column.                  

Output         :  pszData  Null-terminated typed information (only if the function returns PTIRET_OK).

Return         :  PTIRET_OK            Successful data entry.
                  PTIRET_INVPARAM      Invalid parameter passed to the function.
                  PTIRET_NOCONN        The terminal is offline.
                  PTIRET_BUSY          The terminal is busy processing another command.
                  PTIRET_TIMEOUT       No key was pressed during the specified time.
                  PTIRET_CANCEL        User pressed the [CANCEL] key.
                  PTIRET_SECURITYERR   The function was rejected for security reasons
\*===============================================================================================*/
extern void PTI_GetData (LPCSTR pszTerminalId, LPCSTR pszPrompt, LPCSTR pszFormat, UINT16 uiLenMin, 
                         UINT16 uiLenMax, BOOL fFromLeft, BOOL fAlpha, BOOL fMask, 
                         UINT16 uiTimeOutSec, LPSTR pszData, UINT16 uiCaptureLine, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_StartMenu

Description    :  This function starts building an option menu for user selection. 
                  This function returns immediately.
 
Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

Output         :  none.

Return         :  PTIRET_OK      Menu creation was started.
                  PTIRET_NOCONN  The terminal is offline.
                  PTIRET_BUSY    The terminal is busy processing another command.
\*===============================================================================================*/
extern void PTI_StartMenu (LPCSTR pszTerminalId, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_AddMenuOption

Description    :  This function adds an option to the menu that was created through PTI_StartMenu.
                  This function returns immediately.
 
Input          :  pszTerminalId  Terminal unique identifier (null-terminated).
                  pszOption      Null-terminated text message that describes the option to be shown 
                                 on the terminal (maximum: 18 characters).

Output         :  none.

Return         :  PTIRET_OK       Menu creation was started.
                  PTIRET_INVPARAM Invalid parameter passed to the function.
                  PTIRET_NOCONN   The terminal is offline.
                  PTIRET_BUSY     The terminal is busy processing another command.
\*===============================================================================================*/
extern void PTI_AddMenuOption (LPCSTR pszTerminalId, LPCSTR pszOption, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_ExecMenu

Description    :  This function displays on the terminal the option menu that was created through 
                  PTI_StartMenu and PTI_AddMenuOption, and gets a selection from the user.
                  This function is blocking and only returns after an option is selected or an 
                  error occurs.
 
Input          :  pszTerminalId  Terminal unique identifier (null-terminated).
                  pszPrompt      Null-terminated text message to be displayed on top of the menu 
                                 (maximum: 20 characters). Example: CHOOSE AN OPTION:. If NULL 
                                 or empty, the menu is displayed from the top of the display.
                  uiTimeOutSec   Maximum time between two keystrokes, in seconds.
                  puiSelection   Zero-based index of the option that should be pre-selected when 
                                 the menu is first displayed, so that pressing [OK] selects it. 
                                 If puiSelection is not a valid option, none is pre-selected.

Output         :  puiSelection   Zero-based index of the option that was selected by the user 
                                 (only if the function returns PTIRET_OK).

Return         :  PTIRET_OK         Successful menu selection.
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
                  PTIRET_NOCONN     The terminal is offline.
                  PTIRET_BUSY       The terminal is busy processing another command.
                  PTIRET_TIMEOUT    No key was pressed during the specified time.
                  PTIRET_CANCEL     User pressed the [CANCEL] key.
\*===============================================================================================*/
extern void PTI_ExecMenu (LPCSTR pszTerminalId, LPCSTR pszPrompt, UINT16 uiTimeOutSec, 
                          PUINT16 puiSelection, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_Beep

Description    :  This function emits a sound on the terminal. This function returns immediately.
 
Input          :  pszTerminalId  Terminal unique identifier (null-terminated).
                  iType          Sound type, according to table below (PTIBEEP_xxx).

Output         :  none.

Return         :  PTIRET_OK         Successful menu selection.
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
                  PTIRET_NOCONN     The terminal is offline.
                  PTIRET_BUSY       The terminal is busy processing another command.
\*===============================================================================================*/
extern void PTI_Beep (LPCSTR pszTerminalId, INT16 iType, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_Print

Description    :  This function prints one or multiple lines of text on the terminal printer, and 
                  returns immediately. Up to 40 characters per line may be printed, with line breaks 
                  identified by the '\r' character (carriage return, ASCII code 13). Characters beyond 
                  the 40 columns are discarded. The maximum number of supported columns may vary 
                  according to the payment terminal model but, using the default font, at least 40 
                  columns are always supported. If an error occurs during printing, the corresponding 
                  error code is returned by PTI_PrnFeed. PTI_PrnFeed should be called after one or
                  more calls to PTI_Print.

Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

                  pszText        Text to be printed (null-terminated).

Output         :  none.

Return         :  PTIRET_OK            Successful operation.  
                  PTIRET_INVPARAM      Invalid parameter passed to the function.
                  PTIRET_NOCONN        The terminal is offline.
                  PTIRET_BUSY          The terminal is busy processing another command.
                  PTIRET_NOTSUPORTED   Function not supported by the terminal.
\*===============================================================================================*/
extern void PTI_Print (LPCSTR pszTerminalId, LPCSTR pszText, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_PrnFeed

Description    :  This function advances the printer paper a few lines, to allow the user to detach 
                  the receipt.

Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

Output         :  none.

Return         :  PTIRET_OK            Successful operation.
                  PTIRET_INVPARAM      Invalid parameter passed to the function.
                  PTIRET_NOCONN        The terminal is offline.
                  PTIRET_BUSY          The terminal is busy processing another command.
                  PTIRET_PRINTERR      Printer error.
                  PTIRET_NOPAPER       Printer without paper.
                  PTIRET_NOTSUPORTED   Function not supported by the terminal.
\*===============================================================================================*/
extern void PTI_PrnFeed (LPCSTR pszTerminalId, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_EFT_Start

Description    :  The POS software should call this function to start any new transaction.
                  This function returns immediately.

Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

                  iOper          Transaction type (PTITRN_xxx).

Output         :  none.

Return         :  PTIRET_OK         Successful operation.
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
                  PTIRET_NOCONN     The terminal is offline.
                  PTIRET_BUSY       The terminal is busy processing another command.
\*===============================================================================================*/
extern void PTI_EFT_Start (LPCSTR pszTerminalId, INT16 iOper, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_EFT_AddParam

Description    :  The POS software should call this function iteratively after PTI_EFT_Start to set 
                  all available parameters for the transaction. This function returns immediately.

Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

                  iParam         Parameter identifier (PTIPAR_xxx).

                  pszValue       Parameter value (null-terminated).

Output         :  none.

Return         :  PTIRET_OK         Successful operation.
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
                  PTIRET_NOCONN     The terminal is offline.
                  PTIRET_BUSY       The terminal is busy processing another command.
\*===============================================================================================*/
extern void PTI_EFT_AddParam (LPCSTR pszTerminalId, INT16 iParam, LPCSTR pszValue, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_EFT_Exec

Description    :  This function actually performs the transaction, using the parameters that were 
                  previously defined through PTI_EFT_AddParam. This function is blocking, and only 
                  returns after completion (or failure) of the transaction.

Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

Output         :  none.

Return         :  PTIRET_OK         Successful operation (for a sale, it means transaction approval).
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
                  PTIRET_NOCONN     The terminal is offline.
                  PTIRET_BUSY       The terminal is busy processing another command.
                  PTIRET_EFTERR     The EFT operation was performed, but failed.
\*===============================================================================================*/
extern void PTI_EFT_Exec (LPCSTR pszTerminalId, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_EFT_GetInfo

Description    :  The POS software should call this function iteratively to retrieve data elements 
                  from the transaction that was just performed by the terminal. It returns immediately.

Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

                  iInfo          Information identifier to be obtained (PTIINF_xxx).

                  uiBufLen       Size (in bytes) of the buffer pointed to by pszValue.

Output         :  pszValue       Information retrieved (null-terminated).

Return         :  PTIRET_OK         Success, information returned.
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
                  PTIRET_BUFOVRFLW  The data is greater than uiBufLen.
                  PTIRET_NOCONN     The terminal is offline.
                  PTIRET_BUSY       The terminal is busy processing another command.
                  PTIRET_NODATA     Information is not available.
\*===============================================================================================*/
extern void PTI_EFT_GetInfo (LPCSTR pszTerminalId, INT16 iInfo, UINT16 uiBufLen, LPSTR pszValue, 
                             PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_EFT_PrintReceipt

Description    :  This function makes the terminal print the receipts for the last performed 
                  EFT operation. The POS software may choose to:
                     * Use this function to print one or both copies (merchant/cardholder) of 
                       the payment receipt;
                     * Retrieve the receipt contents through PTI_EFT_GetInfo and:
                        * Print one or both of them on a dedicated printer;
                        * E-mail the cardholder copy.
                  Note: a merchant copy should always be printed when PTIINF_CHOLD_VERIF (retrieved 
                        through PTI_EFT_GetInfo) indicates that the cardholder signature is required.

Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

                  iCopies        Sum of values PTIPRN_xxx.

Output         :  pszValue Information retrieved (null-terminated).

Return         :  PTIRET_OK         Success, printing started.
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
                  PTIRET_NOCONN     The terminal is offline.
                  PTIRET_BUSY       The terminal is busy processing another command.
                  PTIRET_NODATA     There is no receipt to be printed.
                  PTIRET_PRINTERR   Printer error.
                  PTIRET_NOPAPER    Printer out of paper.
\*===============================================================================================*/
extern void PTI_EFT_PrintReceipt (LPCSTR pszTerminalId, INT16 iCopies, PINT16 piRet);

/*===============================================================================================*\
Function       :  PTI_EFT_Confirm

Description    :  Any successful EFT operation (PTI_EFT_Exec returned PTIRET_OK) must be confirmed by 
                  the POS software through this function to ensure transaction integrity between all 
                  parties (POS software and tax records, terminal, acquirer, issuer and cardholder).
                  Multiple transactions may be performed simultaneously by several terminals, however, 
                  for each terminal, a transaction should be confirmed before another may be started, 
                  i.e., there may only be one pending transaction at any given time for a terminal.
                  To minimize reversal scenarios, it is recommended that the POS software confirm the 
                  transaction as soon as possible. If PTI_EFT_Exec returns PTIRET_OK and the POS software 
                  does not immediately confirm the transaction, it should immediately store in non-volatile 
                  memory (file) all information necessary to confirm or reverse the transaction in case of
                  a power failure after this time. Events that may lead to a reversal of the transaction are:
                     * Printer failure (when cardholder signature is required);
                     * Goods could not be delivered (dispenser mechanism failure or equivalent);
                     * Power outage (cardholder used an alternative payment method before power came back).

Input          :  pszTerminalId  Terminal unique identifier (null-terminated).

                  iStatus           Final status of the transaction (PTICNF_xxx).

Output         :  none.

Return         :  PTIRET_OK         Confirmation done.
                  PTIRET_INVPARAM   Invalid parameter passed to the function.
\*===============================================================================================*/
extern void PTI_EFT_Confirm (LPCSTR pszTerminalId, INT16 iStatus, PINT16 piRet);


/*===========================================================================*\
Funcao     :   PTI_PrnSymbolCode

Descricao  :   Esta funo imprime a imagem de um cdigo de barras ou
               QRCode.

Entradas   :   pszTerminalId  :  ndice do terminal (iniciado em zero).
               pszMsg         :  Cdigo a ser impresso.
               iSymbology     :  Tipo de cdigo (CODESYMB_xxx)

Saidas     :   no h.

Retorno    :   PTIRET_OK         : Operao bem sucedida.
               PTIRET_INVPARAM   : Parmetro invlido passado  funo.
               PTIRET_NOCONN     : O terminal no est conectado ao Servidor de Automao Comercial.
               PTIRET_BUSY       : O terminal est ocupado processando outro comando.
\*===========================================================================*/
extern void PTI_PrnSymbolCode (LPCSTR pszTerminalId, LPCSTR pszMsg, INT16 iSymbology, 
                     PINT16 piRet);

/*===========================================================================*\
Funcao     :   PTI_ConnectionLoop

Descricao  :   This function allows the POS software to check when a terminal has 
               connected and, if PTIRET_NEWCONN is returned, retrieve additional 
               information from the device.

Entradas   :   none.

Saidas     :   pszTerminalId  : Terminal unique identifier (null-terminated, up to 20 characters).
               pszModel       : Terminal model (null-terminated, up to 20 characters).
               pszMAC         : Terminal MAC address (null-terminated, format XX:XX:XX:XX:XX:XX).
               pszSerNo       : Terminal serial number (null-terminated, up to 25 characters).


Retorno    :   PTIRET_NEWCONN    : New terminal connected.
               PTIRET_NONEWCONN  : No new connections received.
\*===========================================================================*/
extern void PTI_ConnectionLoop ( LPSTR pszTerminalId, LPSTR pszModel, LPSTR pszMAC, 
                                 LPSTR pszSerNo, PINT16 piRet);

#endif /* _PTI_DLL_H */
