/************************ SETIS Automao e Sistemas **************************

 File            : PTI_DLL_SampleTestApp.c
 Project         : Payment Terminal Integration
 Platform        : Win32
 Creation date   : Jul/31/2014
 Author          : Guilherme Eduardo Leite
 Description     : Test Application for PTI Library 
 ================================= HISTORY ====================================
 Date      Author       Change description
 ------------------------------------------------------------------------------
\*****************************************************************************/
#include "stdio.h"
#include "conio.h"
#include "windows.h"
#include "PTI_DLL.h"
#include "Version.h"
#include "PGWebLib.h"
#include <signal.h>

// Structure to load DLL funtions
typedef struct {
    void (*pPTI_Init)               (LPCSTR pszPOS_Company, LPCSTR pszPOS_Version, LPCSTR pszPOS_Capabilities, LPCSTR pszDataFolder, UINT16 uiTCP_Port, UINT16 uiMaxTerminals, LPCSTR pszWaitMsg, UINT16 uiAutoDiscSec, PINT16 piRet);
    void (*pPTI_End)                (void);
    void (*pPTI_CheckStatus)        (LPSTR pszTerminalId, PINT16 piStatus, LPSTR pszModel, LPSTR pszMAC, LPSTR pszSerNo, PINT16 piRet);
    void (*pPTI_Disconnect)         (LPCSTR pszTerminalId, UINT16 uiPwrDelay, PINT16 piRet);
    void (*pPTI_Display)            (LPCSTR pszTerminalId, LPCSTR pszMsg, PINT16 piRet);
    void (*pPTI_Print)              (LPCSTR pszTerminalId, LPCSTR pszText, PINT16 piRet);
    void (*pPTI_PrnFeed)            (LPCSTR pszTerminalId, PINT16 piRet);
    void (*pPTI_EFT_Start)          (LPCSTR pszTerminalId, INT16 iOper, PINT16 piRet);
    void (*pPTI_EFT_AddParam)       (LPCSTR pszTerminalId, INT16 iParam, LPCSTR pszValue, PINT16 piRet);
    void (*pPTI_EFT_Exec)           (LPCSTR pszTerminalId, PINT16 piRet);
    void (*pPTI_EFT_GetInfo)        (LPCSTR pszTerminalId, INT16 iInfo, UINT16 uiBufLen, LPSTR pszValue, PINT16 piRet);
    void (*pPTI_EFT_PrintReceipt)   (LPCSTR pszTerminalId, INT16 iCopies, PINT16 piRet);
    void (*pPTI_EFT_Confirm)        (LPCSTR pszTerminalId, INT16 iStatus, PINT16 piRet);
    void (*pPTI_ConnectionLoop)     (LPSTR pszTerminalId, LPSTR pszModel, LPSTR pszMAC, LPSTR pszSerNo, PINT16 piRet);
} TerminalFunctions;

// Global variables
static HINSTANCE           ghHwLib;
static TerminalFunctions   gstTermFuncs;
static INT16               giTerminalNumber=0;

/////////////////////////
// Auxiliary functions //
/////////////////////////
/*=====================================================================================*\
 Function      :  InputCR

 Description   :  This function is used to replace the character used by server for 
                  newline ('\r') to the standard used by the Windows console applications 
                  ("\r\n").
 
 Input         :  pszSourceStr   :  String to be modified.

 Output        :  pszSourceStr   :  New string.
 
 Retorno       :  None.
\*=====================================================================================*/
static void InputCR(char* pszSourceStr)
{
   int i, j, iSourceLen;
   char cAux, cTemp;

   iSourceLen = strlen(pszSourceStr);

   for( i = 0; i < iSourceLen; i++)
   {
      if( pszSourceStr[i] == '\r' && pszSourceStr[i+1] != '\n')
      {
         cAux = pszSourceStr[i+1];
         pszSourceStr[i+1] = '\n';
         for( j = i+1; j <= iSourceLen; j++)
         {
            cTemp = pszSourceStr[j+1];
            pszSourceStr[j+1] = cAux;
            cAux = cTemp;
         }
         iSourceLen++;
      }
   }
}

/*=====================================================================================*\
 Function      :  PrintReturnDescription

 Description   :  This function prints on the screen the function's return value.
 
 Input         :  iReturnCode :  Function return code (PTIRET_XXX). 

 Output        :  none.
 
 Return        :  none.
\*=====================================================================================*/
static void PrintReturnDescription(INT16 iReturnCode)
{
   switch(iReturnCode)
    {
       case(PTIRET_OK):
          printf("\n\nRetorno = PTIRET_OK");
          break;
       case(PTIRET_INVPARAM):
          printf("\n\nRetorno = PTIRET_INVPARAM");
          break;
       case(PTIRET_NOCONN):
          printf("\n\nRetorno = PTIRET_NOCONN");
          break;
       case(PTIRET_BUSY):
          printf("\n\nRetorno = PTIRET_BUSY");
          break;
       case(PTIRET_NODATA):
          printf("\n\nRetorno = PTIRET_NODATA");
          break;
       case(PTIRET_BUFOVRFLW):
          printf("\n\nRetorno = PTIRET_BUFOVRFLW");
          break;
       case(PTIRET_SOCKETERR):
          printf("\n\nRetorno = PTIRET_SOCKETERR");
          break;
       case(PTIRET_WRITEERR):
          printf("\n\nRetorno = PTIRET_WRITEERR");
          break;
       case(PTIRET_EFTERR):
          printf("\n\nRetorno = PTIRET_EFTERR");
          break;
       case(PTIRET_INTERNALERR):
          printf("\n\nRetorno = PTIRET_INTERNALERR");
          break;
       case(PTIRET_PRINTERR):
          printf("\n\nRetorno = PTIRET_PRINTERR");
          break;
       case(PTIRET_NOPAPER):
          printf("\n\nRetorno = PTIRET_NOPAPER");
          break;
       case(PTIRET_PROTOCOLERR):
          printf("\n\nRetorno = PTIRET_PROTOCOLERR");
          break;
       default:
          printf("\n\nOUTRO ERRO<%d>", iReturnCode);
          break;
    }
   printf("\n");
}

/*=====================================================================================*\
 Function      :  PrintResults

 Description   :  This function prints all the results on the screen
 
 Input         :  pszTerminalID  :  Terminal Identifier.
                  iTrnRet        :  Transaction return code.

 Output        :  none.
 
 Return        :  none.
\*=====================================================================================*/
static void PrintResults(CHAR* pszTerminalID, INT16 iTrnRet)
{
   CHAR  szReturnBuffer[2048];
   INT16 iRet=0;

   UNREFERENCED_PARAMETER(iTrnRet);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_RESULTMSG, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
   {
      InputCR(szReturnBuffer);
      printf("\nPWINFO_RESULTMSGN=\n%s", szReturnBuffer);
   }

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_OPERATION, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_OPERATION=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_REQNUM, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_AUTEXTREF=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_AUTEXTREF, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_AUTEXTREF=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_AUTLOCREF, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_AUTLOCREF=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_AUTHCODE, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_AUTHCODE=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_DATETIME, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_DATETIME=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_TOTAMNT, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_TOTAMNT=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_CURRENCY, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_CURRENCY=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_CARDTYPE, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_CARDTYPE=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_CARDNAME, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_CARDNAME=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_AUTHSYST, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_AUTHSYST=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_AUTMERCHID, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_AUTMERCHID=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_VIRTMERCH, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_VIRTMERCH=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_CARDENTMODE, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_CARDENTMODE=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_CARDPARCPAN, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_CARDPARCPAN=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_CHOLDVERIF, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_CHOLDVERIF=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_AUTRESPCODE, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_AUTRESPCODE=%s", szReturnBuffer);     

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_FINTYPE, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_FINTYPE=%s", szReturnBuffer); 

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_INSTALLMENTS, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_INSTALLMENTS=%s", szReturnBuffer);  

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_PNDVIRTMERCH, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_PNDVIRTMERCH=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_PNDAUTHSYST, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_PNDAUTHSYST=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_PNDAUTEXTREF, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_PNDAUTEXTREF=%s", szReturnBuffer);

   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_PNDAUTLOCREF, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_PNDAUTLOCREF=%s", szReturnBuffer);


   /* COMPROVANTES */
   printf("\n*********** Via Completo ************");
   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_RCPTFULL, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_RCPTFULL=%s", szReturnBuffer);  

   printf("\n*********** Via Estab ************");
   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_RCPTMERCH, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_RCPTMERCH=%s", szReturnBuffer);  

   printf("\n*********** Via Portador ************");
   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_RCPTCHOLDER, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_RCPTCHOLDER=%s", szReturnBuffer);  

   printf("\n*********** Via Reduzida ************");
   gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PWINFO_RCPTCHSHORT, sizeof(szReturnBuffer), szReturnBuffer, &iRet);
   if(iRet==PTIRET_OK)
      printf("\nPWINFO_RCPTCHSHORT=%s", szReturnBuffer);  

   /*
   if( iTrnRet == PTIRET_OK)
   {
      // Merchant receipt
      printf("\n*********** VIA DO LOJISTA ************");
      for(i=0;;i++)
      {
         gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PTIINF_RCPTMERCHANT + i, sizeof(szRcptLine), szRcptLine, &iRet);
         if( iRet == PTIRET_OK)
            printf("\r%s", szRcptLine);
         else
            break;
      }

      // Card holder receipt
      printf("\n*********** VIA DO PORTADOR *********");
      for(i=0;;i++)
      {
         gstTermFuncs.pPTI_EFT_GetInfo(pszTerminalID, PTIINF_RCPTCHOLDER + i, sizeof(szRcptLine), szRcptLine, &iRet);
         if( iRet == PTIRET_OK)
            printf("\r%s", szRcptLine);
         else 
            break;
      }
   }
   */
}

/*=====================================================================================*\
 Function      :  SaleTransaction

 Description   :  This function performs a sale transaction.
 
 Input         :  none. 

 Output        :  none.
 
 Return        :  none.
\*=====================================================================================*/
static void SaleTransaction()
{
   INT16 iRet=0, iTrnRet=0, iStatus=0;
   CHAR szTerminalId[21], szModel[21], szMAC[18], szSerNum[26], szAmount[12];

   printf("\nInforme o ID do ponto de captura do POS:");
   scanf_s(" %[^\n]", szTerminalId, sizeof(szTerminalId) );

   printf("\nInforme o valor da venda, em centavos:");
   scanf_s(" %[^\n]", szAmount, sizeof(szAmount) );

   // Checking terminal status
   gstTermFuncs.pPTI_CheckStatus(szTerminalId, &iStatus, szModel, szMAC, szSerNum, &iRet);

   if( iRet==PTIRET_OK && iStatus==PTISTAT_IDLE)
   {
      // Print terminal information on the screen
      printf("\n\nModelo = %s", szModel);
      printf("\nEndereco MAC = %s", szMAC);
      printf("\nNumero serial = %s", szSerNum);

      // Initializes the transaction 
      gstTermFuncs.pPTI_EFT_Start(szTerminalId, PWOPER_SALE, &iRet);

      // Adds transaction data
      gstTermFuncs.pPTI_EFT_AddParam(szTerminalId, PWINFO_CURRENCY, "986", &iRet);
      gstTermFuncs.pPTI_EFT_AddParam(szTerminalId, PWINFO_TOTAMNT, szAmount, &iRet);
      

      // Executes the transaction
      printf("\n\nRealizando a operacao no terminal POS...");
      gstTermFuncs.pPTI_EFT_Exec(szTerminalId, &iTrnRet);

      // Print the transaction result on the sreen 
      PrintReturnDescription(iTrnRet);

      if(iTrnRet == PTIRET_OK)
      {
         // Confirms the transaction
         printf("\n\nConfirmando a transacao...");
         gstTermFuncs.pPTI_EFT_Confirm(szTerminalId, PTICNF_SUCCESS, &iRet);

         // Print the receipts on the payment terminal
          printf("\n\nImprimindo os comprovantes no terminal POS...");
         gstTermFuncs.pPTI_EFT_PrintReceipt(szTerminalId, PTIPRN_MERCHANT|PTIPRN_CHOLDER, &iRet);
      }

      // Print all the results on the screen
      PrintResults(szTerminalId, iTrnRet);
   }
   else
      PrintReturnDescription(iRet);
}

/*=====================================================================================*\
 Function      :  SaleVoidTransaction

 Description   :  This function performs a sale void transaction.
 
 Input         :  none. 

 Output        :  none.
 
 Return        :  none.
\*=====================================================================================*/
static void SaleVoidTransaction()
{
   INT16 iRet=0, iTrnRet=0, iStatus=0;
   CHAR szTerminalId[21], szModel[21], szMAC[18], szSerNum[26], szAmount[12], szOrigAutExtRef[51];

   printf("\nInforme o ID do ponto de captura do POS:");
   scanf_s(" %[^\n]", szTerminalId, sizeof(szTerminalId) );

   printf("\nInforme o valor da transacao, em centavos:");
   scanf_s(" %[^\n]", szAmount, sizeof(szAmount) );

   printf("\nInforme o numero de referencia da transacao original:");
   scanf_s(" %[^\n]", szOrigAutExtRef, sizeof(szOrigAutExtRef) );

   // Checking terminal status
   gstTermFuncs.pPTI_CheckStatus(szTerminalId, &iStatus, szModel, szMAC, szSerNum, &iRet);

   if( iRet==PTIRET_OK && iStatus==PTISTAT_IDLE)
   {
      // Print terminal information on the screen
      printf("\n\nModelo = %s", szModel);
      printf("\nEndereco MAC = %s", szMAC);
      printf("\nNumero Serial = %s", szSerNum);

      // Initializes the transaction 
      gstTermFuncs.pPTI_EFT_Start(szTerminalId, PWOPER_SALEVOID, &iRet);

      // Adds transaction data
      gstTermFuncs.pPTI_EFT_AddParam(szTerminalId, PWINFO_CURRENCY, "986", &iRet);
      gstTermFuncs.pPTI_EFT_AddParam(szTerminalId, PWINFO_TOTAMNT, szAmount, &iRet);
      gstTermFuncs.pPTI_EFT_AddParam(szTerminalId, PWINFO_TRNORIGNSU, szOrigAutExtRef, &iRet);

      // Executes the transaction
      printf("\n\nRealizando a operacao no terminal POS...");
      gstTermFuncs.pPTI_EFT_Exec(szTerminalId, &iTrnRet);

      // Print the transaction results on the sreen 
      PrintReturnDescription(iTrnRet);

      if(iTrnRet == PTIRET_OK)
      {
         // Confirms the transaction
         printf("\n\nConfirmando a transacao...");
         gstTermFuncs.pPTI_EFT_Confirm(szTerminalId, PTICNF_SUCCESS, &iRet);

         // Print the receipts on the payment terminal
          printf("\n\nImprimindo os comprovantes no terminal POS...");
         gstTermFuncs.pPTI_EFT_PrintReceipt(szTerminalId, PTIPRN_MERCHANT|PTIPRN_CHOLDER, &iRet);
      }

      // Print all the results on the screen
      PrintResults(szTerminalId, iTrnRet);
   }
   else
      PrintReturnDescription(iRet); 
}

/*=====================================================================================*\
 Function      :  Disconnect

 Description   :  This function disconnects a terminal.
 
 Input         :  none. 

 Output        :  none.
 
 Return        :  none.
\*=====================================================================================*/
static void Disconnect()
{
   INT16 iRet=0, iStatus=0;
   CHAR szTerminalId[21], szModel[21], szMAC[18], szSerNum[26];

   printf("\nInforme o ID do ponto de captura do POS:");
   scanf_s(" %[^\n]", szTerminalId, sizeof(szTerminalId) );

   // Checking terminal status
   gstTermFuncs.pPTI_CheckStatus(szTerminalId, &iStatus, szModel, szMAC, szSerNum, &iRet);

   if( iRet==PTIRET_OK && iStatus==PTISTAT_IDLE)
   {
      // Print terminal information on the screen
      printf("\n\nModelo = %s", szModel);
      printf("\nEndereco MAC = %s", szMAC);
      printf("\nNumero Serial = %s", szSerNum);

      // Disconnect
      gstTermFuncs.pPTI_Disconnect(szTerminalId, 0, &iRet);

      // Print the transaction results on the sreen 
      PrintReturnDescription(iRet);
   }
   else
      PrintReturnDescription(iRet); 

}

/*=====================================================================================*\
 Function      :  ConnectionLoop

 Description   :  This function performs a connection check loop.
 
 Input         :  none. 

 Output        :  none.
 
 Return        :  none.
\*=====================================================================================*/
static void ConnectionLoop()
{
   INT16 iRet=0;
   CHAR szTerminalId[21], szModel[21], szMAC[18], szSerNum[26];

   printf ("\n\n\nAGUARDANDO CONEXOES (ESC P/ PARAR)");

   for(;;)
   {
      if(_kbhit() && _getch()==27)
         break;
      printf (".");
      gstTermFuncs.pPTI_ConnectionLoop(szTerminalId, szModel, szMAC, szSerNum, &iRet);
      if(iRet==PTIRET_NEWCONN)
      {
         printf ("\n\n*** TERMINAL %s CONECTADO !!!", szTerminalId);
         printf ("\n    MODELO = %s", szModel);
         printf ("\n    MAC    = %s", szMAC);
         printf ("\n    SERIAL = %s", szSerNum);
         printf ("\n\n\nAGUARDANDO CONEXOES (ESC P/ PARAR)");
      }
      Sleep(500);
   }
}

/*=====================================================================================*\
 Function      :  TestLoop

 Description   :  This function is a loop waiting for the user to select an option.

 Input         :  none 

 Output        :  none.
 
 Return        :  none.
\*=====================================================================================*/
static void TestLoop (void)
{
   INT16   iKey;

   for(;;)
   {
      printf ("\n");
      printf ("\n===================");
      printf ("\nOPCOES:");
      printf ("\n===================");
      printf("\n0 - SAIR");
      printf("\n1 - VENDA");
      printf("\n2 - CANCELAMENTO");
      printf("\n3 - MONITORAR CONEXOES");
      printf("\n4 - DESCONECTAR POS");
      printf ("\n\nESCOLHA A FUNCAO: ");

      do {
         iKey = (INT16)_getch ();
      } while (iKey != '0' && iKey != '1' && iKey != '2' && iKey != '3' && iKey != '4');
      printf ("%c\n", iKey);
   
      switch( iKey)
      {
         case('0'):    
            gstTermFuncs.pPTI_End();
            return;
         case('1'):
            SaleTransaction();
            break;
         case('2'):
            SaleVoidTransaction();
            break;
         case('3'):
            ConnectionLoop();
            break;
         case('4'):
            Disconnect();
            break;
         default:       
            break;;      
      }
   }
}

/////////////////////////
// Exit signal         //
/////////////////////////
void SigBreak_Handler(int n_signal)
{
   UNREFERENCED_PARAMETER(n_signal);
   gstTermFuncs.pPTI_End();
   exit(0);
}

/////////////////////////
// Main function       //
/////////////////////////
int main(int argc, char* argv[])
{
   INT16 iRet = 0;
   UNREFERENCED_PARAMETER (argc);
   UNREFERENCED_PARAMETER (argv);
   signal(SIGBREAK, &SigBreak_Handler);

   printf ("\n");
   printf ("\n==================================================");
   printf ("\n              PTI_DLL.dll - %s", PTIDLLTEST_VERSION);
   printf ("\n");
   printf ("\n            Automacao de Demonstracao");
   printf ("\n==================================================");
   printf ("\n\n");

   ghHwLib = LoadLibrary(TEXT("PTI_DLL.dll"));

   if( ghHwLib == INVALID_HANDLE_VALUE)
   {
      printf ("\n\nErro %ld carregando a biblioteca PTI_DLL.dll\n\n", GetLastError ());
   }
   else
   {
#pragma warning (disable : 4213)
      (FARPROC)gstTermFuncs.pPTI_Init              = GetProcAddress  (ghHwLib, "PTI_Init");
      (FARPROC)gstTermFuncs.pPTI_End               = GetProcAddress  (ghHwLib, "PTI_End");
      (FARPROC)gstTermFuncs.pPTI_CheckStatus       = GetProcAddress  (ghHwLib, "PTI_CheckStatus");
      (FARPROC)gstTermFuncs.pPTI_Disconnect        = GetProcAddress  (ghHwLib, "PTI_Disconnect");
      (FARPROC)gstTermFuncs.pPTI_Display           = GetProcAddress  (ghHwLib, "PTI_Display");
      (FARPROC)gstTermFuncs.pPTI_Print             = GetProcAddress  (ghHwLib, "PTI_Print");
      (FARPROC)gstTermFuncs.pPTI_PrnFeed           = GetProcAddress  (ghHwLib, "PTI_PrnFeed");
      (FARPROC)gstTermFuncs.pPTI_EFT_Start         = GetProcAddress  (ghHwLib, "PTI_EFT_Start");
      (FARPROC)gstTermFuncs.pPTI_EFT_AddParam      = GetProcAddress  (ghHwLib, "PTI_EFT_AddParam");
      (FARPROC)gstTermFuncs.pPTI_EFT_Exec          = GetProcAddress  (ghHwLib, "PTI_EFT_Exec");
      (FARPROC)gstTermFuncs.pPTI_EFT_GetInfo       = GetProcAddress  (ghHwLib, "PTI_EFT_GetInfo");
      (FARPROC)gstTermFuncs.pPTI_EFT_PrintReceipt  = GetProcAddress  (ghHwLib, "PTI_EFT_PrintReceipt");
      (FARPROC)gstTermFuncs.pPTI_EFT_Confirm       = GetProcAddress  (ghHwLib, "PTI_EFT_Confirm");
      (FARPROC)gstTermFuncs.pPTI_ConnectionLoop    = GetProcAddress  (ghHwLib, "PTI_ConnectionLoop");
#pragma warning (default : 4213)
      
      // Try to initialize the integration library
      gstTermFuncs.pPTI_Init("AUTOMACAO DEMO", "AUTOMACAO DEMO v1.02", "63", ".", 9000, 50, 
                             "   AUTOMACAO DEMO\r\r"
                             "TERMINAL PRONTO PARA\r"
                             "A PROXIMA TRANSACAO", 0, &iRet);
      if(iRet)
      {
         printf("\nErro %d em PTI_Init", iRet);
         PrintReturnDescription(iRet);
      }
      else
         TestLoop ();

      FreeLibrary (ghHwLib);
   }

   fflush(stdin);
   printf("\n\nAplicacao finalizada!");
   printf("\n    <Tecle Algo!!> \n\n ");
   _getch();

   return 0;
}
